@@ -103,16 +103,6 @@ or to enable case insensitive matching.
 
 #![deny(missing_docs)]
 
-extern crate aho_corasick;
-extern crate bstr;
-extern crate fnv;
-#[macro_use]
-extern crate log;
-extern crate regex;
-
-#[cfg(feature = "serde1")]
-extern crate serde;
-
 use std::borrow::Cow;
 use std::collections::{BTreeMap, HashMap};
 use std::error::Error as StdError;
@@ -125,9 +115,9 @@ use aho_corasick::AhoCorasick;
 use bstr::{ByteSlice, ByteVec, B};
 use regex::bytes::{Regex, RegexBuilder, RegexSet};
 
-use glob::MatchStrategy;
-pub use glob::{Glob, GlobBuilder, GlobMatcher};
-use pathutil::{file_name, file_name_ext, normalize_path};
+use crate::glob::MatchStrategy;
+pub use crate::glob::{Glob, GlobBuilder, GlobMatcher};
+use crate::pathutil::{file_name, file_name_ext, normalize_path};
 
 mod glob;
 mod pathutil;
@@ -228,7 +218,7 @@ impl ErrorKind {
 }
 
 impl fmt::Display for Error {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.glob {
             None => self.kind.fmt(f),
             Some(ref glob) => {
@@ -239,7 +229,7 @@ impl fmt::Display for Error {
 }
 
 impl fmt::Display for ErrorKind {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match *self {
             ErrorKind::InvalidRecursive
             | ErrorKind::UnclosedClass
@@ -317,7 +307,7 @@ impl GlobSet {
     ///
     /// This takes a Candidate as input, which can be used to amortize the
     /// cost of preparing a path for matching.
-    pub fn is_match_candidate(&self, path: &Candidate) -> bool {
+    pub fn is_match_candidate(&self, path: &Candidate<'_>) -> bool {
         if self.is_empty() {
             return false;
         }
@@ -340,7 +330,7 @@ impl GlobSet {
     ///
     /// This takes a Candidate as input, which can be used to amortize the
     /// cost of preparing a path for matching.
-    pub fn matches_candidate(&self, path: &Candidate) -> Vec<usize> {
+    pub fn matches_candidate(&self, path: &Candidate<'_>) -> Vec<usize> {
         let mut into = vec![];
         if self.is_empty() {
             return into;
@@ -352,7 +342,7 @@ impl GlobSet {
     /// Adds the sequence number of every glob pattern that matches the given
     /// path to the vec given.
     ///
-    /// `into` is is cleared before matching begins, and contains the set of
+    /// `into` is cleared before matching begins, and contains the set of
     /// sequence numbers (in ascending order) after matching ends. If no globs
     /// were matched, then `into` will be empty.
     pub fn matches_into<P: AsRef<Path>>(
@@ -366,7 +356,7 @@ impl GlobSet {
     /// Adds the sequence number of every glob pattern that matches the given
     /// path to the vec given.
     ///
-    /// `into` is is cleared before matching begins, and contains the set of
+    /// `into` is cleared before matching begins, and contains the set of
     /// sequence numbers (in ascending order) after matching ends. If no globs
     /// were matched, then `into` will be empty.
     ///
@@ -374,7 +364,7 @@ impl GlobSet {
     /// cost of preparing a path for matching.
     pub fn matches_candidate_into(
         &self,
-        path: &Candidate,
+        path: &Candidate<'_>,
         into: &mut Vec<usize>,
     ) {
         into.clear();
@@ -423,12 +413,12 @@ impl GlobSet {
                     required_exts.add(i, ext, p.regex().to_owned());
                 }
                 MatchStrategy::Regex => {
-                    debug!("glob converted to regex: {:?}", p);
+                    log::debug!("glob converted to regex: {:?}", p);
                     regexes.add(i, p.regex().to_owned());
                 }
             }
         }
-        debug!(
+        log::debug!(
             "built glob set; {} literals, {} basenames, {} extensions, \
                 {} prefixes, {} suffixes, {} required extensions, {} regexes",
             lits.0.len(),
@@ -456,6 +446,13 @@ impl GlobSet {
     }
 }
 
+impl Default for GlobSet {
+    /// Create a default empty GlobSet.
+    fn default() -> Self {
+        GlobSet::empty()
+    }
+}
+
 /// GlobSetBuilder builds a group of patterns that can be used to
 /// simultaneously match a file path.
 #[derive(Clone, Debug)]
@@ -536,7 +533,7 @@ enum GlobSetMatchStrategy {
 }
 
 impl GlobSetMatchStrategy {
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         use self::GlobSetMatchStrategy::*;
         match *self {
             Literal(ref s) => s.is_match(candidate),
@@ -549,7 +546,11 @@ impl GlobSetMatchStrategy {
         }
     }
 
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         use self::GlobSetMatchStrategy::*;
         match *self {
             Literal(ref s) => s.matches_into(candidate, matches),
@@ -575,12 +576,16 @@ impl LiteralStrategy {
         self.0.entry(lit.into_bytes()).or_insert(vec![]).push(global_index);
     }
 
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         self.0.contains_key(candidate.path.as_bytes())
     }
 
     #[inline(never)]
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         if let Some(hits) = self.0.get(candidate.path.as_bytes()) {
             matches.extend(hits);
         }
@@ -599,7 +604,7 @@ impl BasenameLiteralStrategy {
         self.0.entry(lit.into_bytes()).or_insert(vec![]).push(global_index);
     }
 
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         if candidate.basename.is_empty() {
             return false;
         }
@@ -607,7 +612,11 @@ impl BasenameLiteralStrategy {
     }
 
     #[inline(never)]
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         if candidate.basename.is_empty() {
             return;
         }
@@ -629,7 +638,7 @@ impl ExtensionStrategy {
         self.0.entry(ext.into_bytes()).or_insert(vec![]).push(global_index);
     }
 
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         if candidate.ext.is_empty() {
             return false;
         }
@@ -637,7 +646,11 @@ impl ExtensionStrategy {
     }
 
     #[inline(never)]
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         if candidate.ext.is_empty() {
             return;
         }
@@ -655,7 +668,7 @@ struct PrefixStrategy {
 }
 
 impl PrefixStrategy {
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         let path = candidate.path_prefix(self.longest);
         for m in self.matcher.find_overlapping_iter(path) {
             if m.start() == 0 {
@@ -665,7 +678,11 @@ impl PrefixStrategy {
         false
     }
 
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         let path = candidate.path_prefix(self.longest);
         for m in self.matcher.find_overlapping_iter(path) {
             if m.start() == 0 {
@@ -683,7 +700,7 @@ struct SuffixStrategy {
 }
 
 impl SuffixStrategy {
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         let path = candidate.path_suffix(self.longest);
         for m in self.matcher.find_overlapping_iter(path) {
             if m.end() == path.len() {
@@ -693,7 +710,11 @@ impl SuffixStrategy {
         false
     }
 
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         let path = candidate.path_suffix(self.longest);
         for m in self.matcher.find_overlapping_iter(path) {
             if m.end() == path.len() {
@@ -707,7 +728,7 @@ impl SuffixStrategy {
 struct RequiredExtensionStrategy(HashMap<Vec<u8>, Vec<(usize, Regex)>, Fnv>);
 
 impl RequiredExtensionStrategy {
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         if candidate.ext.is_empty() {
             return false;
         }
@@ -725,7 +746,11 @@ impl RequiredExtensionStrategy {
     }
 
     #[inline(never)]
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         if candidate.ext.is_empty() {
             return;
         }
@@ -746,11 +771,15 @@ struct RegexSetStrategy {
 }
 
 impl RegexSetStrategy {
-    fn is_match(&self, candidate: &Candidate) -> bool {
+    fn is_match(&self, candidate: &Candidate<'_>) -> bool {
         self.matcher.is_match(candidate.path.as_bytes())
     }
 
-    fn matches_into(&self, candidate: &Candidate, matches: &mut Vec<usize>) {
+    fn matches_into(
+        &self,
+        candidate: &Candidate<'_>,
+        matches: &mut Vec<usize>,
+    ) {
         for i in self.matcher.matches(candidate.path.as_bytes()) {
             matches.push(self.map[i]);
         }
@@ -833,8 +862,8 @@ impl RequiredExtensionStrategyBuilder {
 
 #[cfg(test)]
 mod tests {
-    use super::GlobSetBuilder;
-    use glob::Glob;
+    use super::{GlobSet, GlobSetBuilder};
+    use crate::glob::Glob;
 
     #[test]
     fn set_works() {
@@ -863,4 +892,11 @@ mod tests {
         assert!(!set.is_match(""));
         assert!(!set.is_match("a"));
     }
+
+    #[test]
+    fn default_set_is_empty_works() {
+        let set: GlobSet = Default::default();
+        assert!(!set.is_match(""));
+        assert!(!set.is_match("a"));
+    }
 }
